package MT::Plugin::BusinessYear;
use strict;
use MT;
use MT::Template::Context;

use MT;
use MT::Entry;
use MT::TemplateMap;
use MT::Template;
use MT::WeblogPublisher;
use File::Basename;
use MT::FileInfo;
use MT::Util qw(start_end_month);

require MT::Plugin;

my $VERSION = '0.1';

use base qw( MT::Plugin );

###################################### Init Plugin #####################################

@MT::Plugin::BusinessYear::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::BusinessYear({
    name => 'BusinessYear',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    'version' => $VERSION,
    description => 'Make Business year archive.',
    system_config_template => 'system_config.tmpl',
    settings => new MT::PluginSettings([
        ['business_year', { Default => 4 }],
    ]),
});

MT->add_plugin($plugin);

####################################### Callbacks ######################################

MT->add_callback('build_file_filter', 1, $plugin, \&_business_year);
MT::Template::Context->add_tag(RebuildBusinessYearArchives => \&_rebuild_business_year_archives);

######################################### Main #########################################

sub _business_year {
	my ($eh, %args) = @_;
	my $tmpl = $args{'Template'};
	if ($tmpl->name eq 'business_year') {
	    return 0;
	}
    return 1;
}

sub _rebuild_business_year_archives {
	my ($ctx, $args, $cond) = @_;
    my $blog = $ctx->stash('blog');
    my $blog_id = $blog->id;
    my $at = 'Yearly';
    my ($year, $start_month, $end_month, $begin_yearly, $start, $end, $start_year, $end_year);
    $start_month = $plugin->get_config_value('business_year');
    my $oldest_entry = MT::Entry->load( {blog_id => $blog_id},
                                        {sort => "authored_on", limit => 1, direction => "ascend"});
    my $newest_entry = MT::Entry->load( {blog_id => $blog_id},
                                        {sort => "authored_on", limit => 1, direction => "descend"});    
    my $oldest_ts = $oldest_entry->authored_on;
    my $newest_ts = $newest_entry->authored_on;
    my $oldest_yyyy = substr($oldest_ts, 0, 4);
    my $newest_yyyy = substr($newest_ts, 0, 4);
    my $oldest_mm = substr($oldest_ts, 4, 2);
    my $newest_mm = substr($newest_ts, 4, 2);
    if ($start_month == 1) {
        $end_month = 12;
    } else {
        $end_month = $start_month - 1;
    }
    unless ($oldest_mm > $end_month) {
        $oldest_yyyy = $oldest_yyyy -1;
    }
    unless ($newest_mm > $end_month) {
        $newest_yyyy = $newest_yyyy -1;
    }
    my $tmpl = MT::Template->load({ blog_id => $blog_id,
                                    name => 'business_year'})
                                        or die 'Can\'t load template.';
    my $map = MT::TemplateMap->load({   blog_id => $blog_id,
                                        template_id => $tmpl->id})
                                           or die 'Can\'t find template map.';
    my $page_tmpl = $tmpl->text;
    $start_month = sprintf("%02d", $start_month);
    $end_month = sprintf("%02d", $end_month);   
    for ($oldest_yyyy .. $newest_yyyy) {
        $year = $_;
        if ($start_month == 1) {
            $end = $year;
        } else {
            $end = $year + 1;
        }
        $start = $year.$start_month.'01000000';
        $end .= $end_month.'01000000';
        my @st_end = &start_end_month($end);
        $end = $st_end[1];
        $begin_yearly = $year.'0101000000';                             
        my $ctx = MT::Template::Context->new;
        $ctx->stash('blog', $blog);
        $ctx->stash('blog_id', $blog_id);
        $ctx->{archive_type} = $at;
        $ctx->{current_archive_type} = $at;
        $ctx->{current_timestamp} = $start;
        $ctx->{current_timestamp_end} = $end;
        my $build = MT::Builder->new;
        my $tokens = $build->compile($ctx, $page_tmpl)
                                    or die 'Can\'t compile template.';
        my $html = $build->build($ctx, $tokens)
                                    or die 'Can\'t build page.';
        my $pub = MT::WeblogPublisher->new;
        my $file = $pub->archive_file_for(undef, $blog, $at, undef, $map, $begin_yearly, undef);       
        my $url = $blog->archive_url;
        $url .= '/' unless $url =~ m|/$|;
        $url .= $file;
        my $archive_root = $blog->archive_path;
        $file = File::Spec->catfile($archive_root, $file);
        my $fmgr = $blog->file_mgr;
        my $finfo = MT::FileInfo->set_info_for_url(
                        $url, $file, $at,
                        {
                            Blog        => $blog_id,
                            TemplateMap => $map->id,
                            Template    => $map->template_id,
                            StartDate => $start,
                        }
                      )
                      || die "Couldn't create FileInfo because "
                      . MT::FileInfo->errstr();
        my $orig_html = $html;
        MT->run_callbacks(
            'build_page',
            Context      => $ctx,
            context      => $ctx,
            ArchiveType  => $at,
            archive_type => $at,
            TemplateMap  => $map,
            template_map => $map,
            Blog         => $blog,
            blog         => $blog,
            Entry        => undef,
            entry        => undef,
            FileInfo     => $finfo,
            file_info    => $finfo,
            PeriodStart  => $start,
            period_start => $start,
            Category     => undef,
            category     => undef,
            RawContent   => \$orig_html,
            raw_content  => \$orig_html,
            Content      => \$html,
            content      => \$html,
            BuildResult  => \$orig_html,
            build_result => \$orig_html,
            Template     => $tmpl,
            template     => $tmpl,
            File         => $file,
            file         => $file
        );
        return '' unless $fmgr->content_is_updated($file, \$html);
        my $path = dirname($file);
        $path =~ s!/$!! unless $path eq '/';
        unless ($fmgr->exists($path)) {
            $fmgr->mkpath($path);
        }
        my $temp_file = "$file.new";
        defined($fmgr->put_data($html, $temp_file));
        $fmgr->rename($temp_file, $file);
        MT->run_callbacks(
            'build_file',
            Context      => $ctx,
            context      => $ctx,
            ArchiveType  => $at,
            archive_type => $at,
            TemplateMap  => $map,
            template_map => $map,
            FileInfo     => $finfo,
            file_info    => $finfo,
            Blog         => $blog,
            blog         => $blog,
            Entry        => undef,
            entry        => undef,
            PeriodStart  => $start,
            period_start => $start,
            RawContent   => \$orig_html,
            raw_content  => \$orig_html,
            Content      => \$html,
            content      => \$html,
            BuildResult  => \$orig_html,
            build_result => \$orig_html,
            Template     => $tmpl,
            template     => $tmpl,
            Category     => undef,
            category     => undef,
            File         => $file,
            file         => $file
        );
    }
    return '';
}

1;